import React, { memo, useState, useEffect } from 'react';
import { View, Text } from 'react-native';
import Slider from './components/Slider';
import PlayPauseButton from './components/PlayPauseButton';
import { Audio } from 'expo-av';
import { millisToMinutesAndSeconds } from './utils/millisToMinutesAndSeconds';
import { getPlayedPercentage } from './utils/getPlayedPercentage';
import { getPlayedPosition } from './utils/getPlayerPosition';
import { styles } from './styles';
const AudioComponent = props => {
  const [soundState, setSoundState] = useState();
  const [playbackStatus, setPlaybackStatus] = useState();
  async function playSound() {
    await (soundState === null || soundState === void 0 ? void 0 : soundState.playAsync());
  }
  async function pauseSound() {
    await (soundState === null || soundState === void 0 ? void 0 : soundState.pauseAsync());
  }
  const onPlayPausePress = () => {
    playbackStatus !== null && playbackStatus !== void 0 && playbackStatus.isPlaying ? pauseSound() : playSound();
  };
  const onSliderChange = value => {
    soundState === null || soundState === void 0 ? void 0 : soundState.setPositionAsync(getPlayedPosition({
      duration: playbackStatus === null || playbackStatus === void 0 ? void 0 : playbackStatus.durationMillis,
      playedPercentage: value
    }));
  };
  useEffect(() => {
    (async () => {
      var _props$audio;
      const {
        sound
      } = await Audio.Sound.createAsync({
        uri: (_props$audio = props.audio) === null || _props$audio === void 0 ? void 0 : _props$audio.fileUrl
      });
      sound.setOnPlaybackStatusUpdate(status => setPlaybackStatus(status));
      setSoundState(sound);
    })();
  }, []);
  useEffect(() => {
    return soundState ? () => {
      soundState.unloadAsync();
    } : undefined;
  }, [soundState]);
  return /*#__PURE__*/React.createElement(View, {
    style: styles.audio
  }, /*#__PURE__*/React.createElement(PlayPauseButton, {
    isPlaying: !!(playbackStatus !== null && playbackStatus !== void 0 && playbackStatus.isPlaying),
    onPress: onPlayPausePress
  }), /*#__PURE__*/React.createElement(Text, {
    style: styles.duration
  }, millisToMinutesAndSeconds(playbackStatus === null || playbackStatus === void 0 ? void 0 : playbackStatus.positionMillis), " /", ' ', millisToMinutesAndSeconds(playbackStatus === null || playbackStatus === void 0 ? void 0 : playbackStatus.durationMillis)), /*#__PURE__*/React.createElement(Slider, {
    onValueChange: onSliderChange,
    value: getPlayedPercentage({
      currentTime: playbackStatus === null || playbackStatus === void 0 ? void 0 : playbackStatus.positionMillis,
      duration: playbackStatus === null || playbackStatus === void 0 ? void 0 : playbackStatus.durationMillis
    })
  }));
};
export default /*#__PURE__*/memo(AudioComponent);
//# sourceMappingURL=index.js.map