import React, { useContext, useEffect, useCallback, useRef, useState } from 'react';
import { Linking } from 'react-native';
import { SocketContextProvider, SocketContext } from './Context';
import { From, MessageTypes, CarouselDestinationTypes } from '../../types/Message';
// import { useAsyncStorage } from '../AsyncStorage';
import { useChatConfigurations } from '../ChatConfigurations';
import { getHourAndMinutes } from '../../utils/getHourAndMinutes';
import { SOCKET_URL } from '../../constants';
import api from '../../services/api';
// import { useDebounce } from '../../hooks/useDebounce';

const botChannel = 'WebChat';
// const TWO_SECONDS = 2000;

function SocketContextComponent(_ref) {
  let {
    children,
    botId,
    params
  } = _ref;
  const clientRef = useRef(null);
  const isFirstRender = useRef(true);

  // const { getItemsAsync, clearAsync, saveDataAsync } = useAsyncStorage();
  const {
    fetchBotAndUpdateConfigs
  } = useChatConfigurations();
  const [storageState, setStorageState] = useState(null);

  // const debouncedMessages = useDebounce(storageState?.messages, TWO_SECONDS);

  const subscribe = async () => {
    const {
      data
    } = await api.post('/webchat/subscribe', {
      sessionId: storageState === null || storageState === void 0 ? void 0 : storageState.sessionId,
      botId,
      channel: 'webchat',
      botChannel,
      parameters: JSON.stringify(params || '[]')
    });
    return data;
  };
  const addMessage = newMessage => {
    setStorageState(prevState => {
      var _prevState$messages, _prevState$messages$f;
      const newMessages = [...((prevState === null || prevState === void 0 ? void 0 : (_prevState$messages = prevState.messages) === null || _prevState$messages === void 0 ? void 0 : (_prevState$messages$f = _prevState$messages.filter) === null || _prevState$messages$f === void 0 ? void 0 : _prevState$messages$f.call(_prevState$messages, m => m.type !== MessageTypes.TYPING)) ?? []), {
        ...newMessage,
        hour: getHourAndMinutes()
      }];
      return {
        messages: newMessages,
        sessionId: prevState === null || prevState === void 0 ? void 0 : prevState.sessionId
      };
    });
  };
  const handleSubmitMessage = useCallback(async _ref2 => {
    let {
      message,
      type,
      ext
    } = _ref2;
    const isMedia = type !== MessageTypes.TEXT;
    try {
      addMessage({
        from: From.USER,
        message,
        type
      });
      await api.post('/webchat/message', {
        botId,
        message,
        isMedia,
        ext,
        sessionId: storageState === null || storageState === void 0 ? void 0 : storageState.sessionId,
        botChannel,
        isPreview: false
      });
    } catch (error) {
      console.log('Error sending message', error);
    }
  }, [storageState === null || storageState === void 0 ? void 0 : storageState.sessionId]);
  const onClientMessage = msg => {
    addMessage({
      ...msg,
      from: From.BOT
    });
  };
  async function startConversation() {
    const subscribeResponse = await subscribe();
    if (!(storageState !== null && storageState !== void 0 && storageState.sessionId) || subscribeResponse.sessionId !== (storageState === null || storageState === void 0 ? void 0 : storageState.sessionId)) {
      const newData = {
        sessionId: subscribeResponse.sessionId,
        messages: (storageState === null || storageState === void 0 ? void 0 : storageState.messages) || []
      };
      setStorageState(newData);
      // await saveDataAsync(newData);
    }

    return subscribeResponse;
  }
  const restartConversation = async () => {
    setStorageState(null);

    // await clearAsync();
    await startConversation();
  };

  // useEffect(() => {
  //   (async () => {
  //     if (debouncedMessages?.length && storageState?.sessionId) {
  //       await saveDataAsync(storageState as StorageState);
  //     }
  //   })();
  // }, [debouncedMessages]);

  // useEffect(() => {
  //   getItemsAsync()
  //     .then((stored) => {
  //       setStorageState(stored);
  //     })
  //     .catch(console.log);
  // }, []);

  useEffect(() => {
    const client = new WebSocket(SOCKET_URL);
    if (!clientRef.current) {
      clientRef.current = client;
      client.onmessage = msg => {
        try {
          const serverResponse = JSON.parse(msg.data);
          if (serverResponse.action === 'message') {
            onClientMessage(serverResponse.data);
          }
        } catch (error) {
          console.log('Error treating received message', error);
        }
      };
    }
    return () => {
      client.close();
      clientRef.current = null;
    };
  }, [SOCKET_URL]);
  useEffect(() => {
    (async () => {
      await fetchBotAndUpdateConfigs(botId);
    })();
  }, []);
  useEffect(() => {
    if (isFirstRender.current) {
      isFirstRender.current = false;
      return;
    }
    startConversation();
  }, []);
  const handleOpenLink = async url => {
    await Linking.openURL(url);
  };
  const handleCarouselButtonClick = async _ref3 => {
    let {
      clickedButton,
      clickedCard
    } = _ref3;
    const {
      destination
    } = clickedButton;
    if ((destination === null || destination === void 0 ? void 0 : destination.type) === CarouselDestinationTypes.URL) {
      handleOpenLink(destination === null || destination === void 0 ? void 0 : destination.value);
    }
    if ((destination === null || destination === void 0 ? void 0 : destination.type) === CarouselDestinationTypes.PHONE) {
      var _destination$value;
      const treated = destination === null || destination === void 0 ? void 0 : (_destination$value = destination.value) === null || _destination$value === void 0 ? void 0 : _destination$value.replace(/[^A-Z0-9]/gi, '');
      handleOpenLink(`tel:+55${treated}`);
    }
    await api.post('/webchat/action', {
      botId: botId,
      action: {
        type: MessageTypes.CAROUSEL,
        data: {
          clickedButton,
          clickedCard
        }
      },
      sessionId: storageState === null || storageState === void 0 ? void 0 : storageState.sessionId
    });
  };
  return /*#__PURE__*/React.createElement(SocketContextProvider, {
    value: {
      handleCarouselButtonClick,
      handleSubmitMessage,
      messages: (storageState === null || storageState === void 0 ? void 0 : storageState.messages) || [],
      restartConversation
    }
  }, children);
}
export default SocketContextComponent;
export function useSocketContext() {
  return useContext(SocketContext);
}
//# sourceMappingURL=Component.js.map